#include "Stepper_click_objects.h"
#include "Stepper_click_resources.h"
#include "built_in.h"


// TFT module connections
unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RST at GPIOE_ODR.B8;
sbit TFT_RS at GPIOE_ODR.B12;
sbit TFT_CS at GPIOE_ODR.B15;
sbit TFT_RD at GPIOE_ODR.B10;
sbit TFT_WR at GPIOE_ODR.B11;
sbit TFT_BLED at GPIOE_ODR.B9;
// End TFT module connections

// Touch Panel module connections
sbit DriveX_Left at GPIOB_ODR.B1;
sbit DriveX_Right at GPIOB_ODR.B8;
sbit DriveY_Up at GPIOB_ODR.B9;
sbit DriveY_Down at GPIOB_ODR.B0;
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
const ADC_THRESHOLD = 1500;
char PenDown;
void *PressedObject;
int PressedObjectType;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

int _object_count;
unsigned short object_pressed;
TButton *local_button;
TButton *exec_button;
int button_order;
TLabel *local_label;
TLabel *exec_label;
int label_order;
TProgressBar *local_progress_bar;
TProgressBar *exec_progress_bar;
int progress_bar_order;


void Init_ADC() {
  ADC_Set_Input_Channel(_ADC_CHANNEL_8 | _ADC_CHANNEL_9);
  ADC1_Init();
  Delay_ms(100);
}
static void InitializeTouchPanel() {
  Init_ADC();
  TFT_Init_ILI9341_8bit(320, 240);

  TP_TFT_Init(320, 240, 8, 9);                                  // Initialize touch panel
  TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold

  PenDown = 0;
  PressedObject = 0;
  PressedObjectType = -1;
}


/////////////////////////
  TScreen*  CurrentScreen;

  TScreen                Screen1;
  TButton               Button1;
char Button1_Caption[11] = "2038 Steps";

  TButton               Button2;
char Button2_Caption[10] = "Full step";

  TButton               Button3;
char Button3_Caption[9] = "1/4 Step";

  TButton               Button4;
char Button4_Caption[9] = "1/2 Step";

  TProgressBar                 ProgressBar1;
char ProgressBar1_Caption[6] = "50";

  TButton               Button5;
char Button5_Caption[17] = "Change direction";

  TButton               Button6;
char Button6_Caption[9] = "1/8 Step";

  TLabel                 Label1;
char Label1_Caption[53] = "You can only change settings when motor isn\'t moving";

  TLabel                 Label2;
char Label2_Caption[22] = "Made by: Ivan Palijan";

  TButton                * const code Screen1_Buttons[6]=
         {
         &Button1,             
         &Button2,             
         &Button3,             
         &Button4,             
         &Button5,             
         &Button6              
         };
  TLabel                 * const code Screen1_Labels[2]=
         {
         &Label1,              
         &Label2               
         };
  TProgressBar           * const code Screen1_ProgressBars[1]=
         {
         &ProgressBar1         
         };



static void InitializeObjects() {
  Screen1.Color                     = 0x1A13;
  Screen1.Width                     = 320;
  Screen1.Height                    = 240;
  Screen1.ButtonsCount              = 6;
  Screen1.Buttons                   = Screen1_Buttons;
  Screen1.LabelsCount               = 2;
  Screen1.Labels                    = Screen1_Labels;
  Screen1.ProgressBarsCount         = 1;
  Screen1.ProgressBars              = Screen1_ProgressBars;
  Screen1.ObjectsCount              = 9;


  Button1.OwnerScreen     = &Screen1;
  Button1.Order           = 0;
  Button1.Left            = 6;
  Button1.Top             = 8;
  Button1.Width           = 86;
  Button1.Height          = 54;
  Button1.Pen_Width       = 1;
  Button1.Pen_Color       = 0x0000;
  Button1.Visible         = 1;
  Button1.Active          = 1;
  Button1.Transparent     = 1;
  Button1.Caption         = Button1_Caption;
  Button1.TextAlign       = _taCenter;
  Button1.FontName        = Tahoma11x13_Regular;
  Button1.PressColEnabled = 1;
  Button1.Font_Color      = 0x0000;
  Button1.Gradient        = 1;
  Button1.Gradient_Orientation = 0;
  Button1.Gradient_Start_Color = 0xFFFF;
  Button1.Gradient_End_Color = 0xC618;
  Button1.Color           = 0xC618;
  Button1.Press_Color     = 0xE71C;
  Button1.OnUpPtr         = 0;
  Button1.OnDownPtr       = 0;
  Button1.OnClickPtr      = Button1OnClick;
  Button1.OnPressPtr      = 0;

  Button2.OwnerScreen     = &Screen1;
  Button2.Order           = 1;
  Button2.Left            = 7;
  Button2.Top             = 105;
  Button2.Width           = 86;
  Button2.Height          = 54;
  Button2.Pen_Width       = 1;
  Button2.Pen_Color       = 0x0000;
  Button2.Visible         = 1;
  Button2.Active          = 1;
  Button2.Transparent     = 1;
  Button2.Caption         = Button2_Caption;
  Button2.TextAlign       = _taCenter;
  Button2.FontName        = Tahoma11x13_Regular;
  Button2.PressColEnabled = 1;
  Button2.Font_Color      = 0x0000;
  Button2.Gradient        = 1;
  Button2.Gradient_Orientation = 0;
  Button2.Gradient_Start_Color = 0xFFFF;
  Button2.Gradient_End_Color = 0x07E0;
  Button2.Color           = 0xC618;
  Button2.Press_Color     = 0xE71C;
  Button2.OnUpPtr         = 0;
  Button2.OnDownPtr       = 0;
  Button2.OnClickPtr      = Button2OnClick;
  Button2.OnPressPtr      = 0;

  Button3.OwnerScreen     = &Screen1;
  Button3.Order           = 2;
  Button3.Left            = 8;
  Button3.Top             = 163;
  Button3.Width           = 86;
  Button3.Height          = 54;
  Button3.Pen_Width       = 1;
  Button3.Pen_Color       = 0x0000;
  Button3.Visible         = 1;
  Button3.Active          = 1;
  Button3.Transparent     = 1;
  Button3.Caption         = Button3_Caption;
  Button3.TextAlign       = _taCenter;
  Button3.FontName        = Tahoma11x13_Regular;
  Button3.PressColEnabled = 1;
  Button3.Font_Color      = 0x0000;
  Button3.Gradient        = 1;
  Button3.Gradient_Orientation = 0;
  Button3.Gradient_Start_Color = 0xFFFF;
  Button3.Gradient_End_Color = 0xC618;
  Button3.Color           = 0xC618;
  Button3.Press_Color     = 0xE71C;
  Button3.OnUpPtr         = 0;
  Button3.OnDownPtr       = 0;
  Button3.OnClickPtr      = Button3OnClick;
  Button3.OnPressPtr      = 0;

  Button4.OwnerScreen     = &Screen1;
  Button4.Order           = 3;
  Button4.Left            = 96;
  Button4.Top             = 105;
  Button4.Width           = 86;
  Button4.Height          = 54;
  Button4.Pen_Width       = 1;
  Button4.Pen_Color       = 0x0000;
  Button4.Visible         = 1;
  Button4.Active          = 1;
  Button4.Transparent     = 1;
  Button4.Caption         = Button4_Caption;
  Button4.TextAlign       = _taCenter;
  Button4.FontName        = Tahoma11x13_Regular;
  Button4.PressColEnabled = 1;
  Button4.Font_Color      = 0x0000;
  Button4.Gradient        = 1;
  Button4.Gradient_Orientation = 0;
  Button4.Gradient_Start_Color = 0xFFFF;
  Button4.Gradient_End_Color = 0xC618;
  Button4.Color           = 0xC618;
  Button4.Press_Color     = 0xE71C;
  Button4.OnUpPtr         = 0;
  Button4.OnDownPtr       = 0;
  Button4.OnClickPtr      = Button4OnClick;
  Button4.OnPressPtr      = 0;

  ProgressBar1.OwnerScreen     = &Screen1;
  ProgressBar1.Order           = 4;
  ProgressBar1.Left            = 197;
  ProgressBar1.Top             = 26;
  ProgressBar1.Width           = 100;
  ProgressBar1.Height          = 17;
  ProgressBar1.Pen_Width       = 1;
  ProgressBar1.Pen_Color       = 0x0000;
  ProgressBar1.Visible         = 1;
  ProgressBar1.Transparent     = 1;
  ProgressBar1.Caption         = ProgressBar1_Caption;
  ProgressBar1.FontName        = Tahoma11x13_Regular;
  ProgressBar1.Font_Color      = 0x0000;
  ProgressBar1.Gradient        = 1;
  ProgressBar1.Gradient_Orientation = 0;
  ProgressBar1.Gradient_Start_Color = 0xFFFF;
  ProgressBar1.Gradient_End_Color = 0x07E0;
  ProgressBar1.Color           = 0xA65E;
  ProgressBar1.Background_Color = 0x8410;
  ProgressBar1.Min             = 0;
  ProgressBar1.Max             = 2036;
  ProgressBar1.Position        = 50;
  ProgressBar1.Show_Position   = 1;
  ProgressBar1.Show_Percentage = 0;
  ProgressBar1.Smooth          = 1;
  ProgressBar1.Rounded         = 0;
  ProgressBar1.Corner_Radius   = 1;

  Button5.OwnerScreen     = &Screen1;
  Button5.Order           = 5;
  Button5.Left            = 96;
  Button5.Top             = 8;
  Button5.Width           = 86;
  Button5.Height          = 54;
  Button5.Pen_Width       = 1;
  Button5.Pen_Color       = 0x0000;
  Button5.Visible         = 1;
  Button5.Active          = 1;
  Button5.Transparent     = 1;
  Button5.Caption         = Button5_Caption;
  Button5.TextAlign       = _taCenter;
  Button5.FontName        = Tahoma11x13_Regular;
  Button5.PressColEnabled = 1;
  Button5.Font_Color      = 0x0000;
  Button5.Gradient        = 1;
  Button5.Gradient_Orientation = 0;
  Button5.Gradient_Start_Color = 0xFFFF;
  Button5.Gradient_End_Color = 0xC618;
  Button5.Color           = 0xC618;
  Button5.Press_Color     = 0xE71C;
  Button5.OnUpPtr         = 0;
  Button5.OnDownPtr       = 0;
  Button5.OnClickPtr      = Button5OnClick;
  Button5.OnPressPtr      = 0;

  Button6.OwnerScreen     = &Screen1;
  Button6.Order           = 6;
  Button6.Left            = 96;
  Button6.Top             = 163;
  Button6.Width           = 86;
  Button6.Height          = 54;
  Button6.Pen_Width       = 1;
  Button6.Pen_Color       = 0x0000;
  Button6.Visible         = 1;
  Button6.Active          = 1;
  Button6.Transparent     = 1;
  Button6.Caption         = Button6_Caption;
  Button6.TextAlign       = _taCenter;
  Button6.FontName        = Tahoma11x13_Regular;
  Button6.PressColEnabled = 1;
  Button6.Font_Color      = 0x0000;
  Button6.Gradient        = 1;
  Button6.Gradient_Orientation = 0;
  Button6.Gradient_Start_Color = 0xFFFF;
  Button6.Gradient_End_Color = 0xC618;
  Button6.Color           = 0xC618;
  Button6.Press_Color     = 0xE71C;
  Button6.OnUpPtr         = 0;
  Button6.OnDownPtr       = 0;
  Button6.OnClickPtr      = Button6OnClick;
  Button6.OnPressPtr      = 0;

  Label1.OwnerScreen     = &Screen1;
  Label1.Order           = 7;
  Label1.Left            = 18;
  Label1.Top             = 77;
  Label1.Width           = 274;
  Label1.Height          = 15;
  Label1.Visible         = 1;
  Label1.Active          = 1;
  Label1.Caption         = Label1_Caption;
  Label1.FontName        = Tahoma11x13_Regular;
  Label1.Font_Color      = 0xF000;
  Label1.OnUpPtr         = 0;
  Label1.OnDownPtr       = 0;
  Label1.OnClickPtr      = 0;
  Label1.OnPressPtr      = 0;

  Label2.OwnerScreen     = &Screen1;
  Label2.Order           = 8;
  Label2.Left            = 206;
  Label2.Top             = 219;
  Label2.Width           = 108;
  Label2.Height          = 15;
  Label2.Visible         = 1;
  Label2.Active          = 1;
  Label2.Caption         = Label2_Caption;
  Label2.FontName        = Tahoma11x13_Regular;
  Label2.Font_Color      = 0xF000;
  Label2.OnUpPtr         = 0;
  Label2.OnDownPtr       = 0;
  Label2.OnClickPtr      = 0;
  Label2.OnPressPtr      = 0;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetButton(index)              CurrentScreen->Buttons[index]
#define GetLabel(index)               CurrentScreen->Labels[index]
#define GetProgressBar(index)         CurrentScreen->ProgressBars[index]

void DeleteTrailingSpaces(char* str){
  char counter;
  while(1) {
  if (str[0] == ' ') {
    for(counter = 0; counter < strlen(str); counter++) {
      str[counter] = str[counter+1];
    }
  }
  else
    break;
  }
}


void DrawButton(TButton *Abutton) {
  if (Abutton->Visible == 1) {
    if (object_pressed == 1) {
      object_pressed = 0;
      TFT_Set_Brush(Abutton->Transparent, Abutton->Press_Color, Abutton->Gradient, Abutton->Gradient_Orientation, Abutton->Gradient_End_Color, Abutton->Gradient_Start_Color);
    }
    else {
      TFT_Set_Brush(Abutton->Transparent, Abutton->Color, Abutton->Gradient, Abutton->Gradient_Orientation, Abutton->Gradient_Start_Color, Abutton->Gradient_End_Color);
    }
    TFT_Set_Pen(Abutton->Pen_Color, Abutton->Pen_Width);
    TFT_Rectangle(Abutton->Left, Abutton->Top, Abutton->Left + Abutton->Width - 1, Abutton->Top + Abutton->Height - 1);
    TFT_Set_Font(Abutton->FontName, Abutton->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text_Return_Pos(Abutton->Caption, Abutton->Left, Abutton->Top);
    if (Abutton->TextAlign == _taLeft)
      TFT_Write_Text(Abutton->Caption, Abutton->Left + 4, (Abutton->Top + ((Abutton->Height - caption_height) / 2)));
    else if (Abutton->TextAlign == _taCenter)
      TFT_Write_Text(Abutton->Caption, (Abutton->Left + (Abutton->Width - caption_length) / 2), (Abutton->Top + ((Abutton->Height - caption_height) / 2)));
    else if (Abutton->TextAlign == _taRight)
      TFT_Write_Text(Abutton->Caption, Abutton->Left + (Abutton->Width - caption_length - 4), (Abutton->Top + (Abutton->Height - caption_height) / 2));
  }
}

void DrawLabel(TLabel *ALabel) {
  if (ALabel->Visible == 1) {
    TFT_Set_Font(ALabel->FontName, ALabel->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text(ALabel->Caption, ALabel->Left, ALabel->Top);
  }
}

void UpdatePBPosition(TProgressBar *AProgressBar) {
  int move_offset;
  unsigned long locPosition = 0;
  unsigned cnt = 0;
  char tmpStr[8];
  unsigned int tmpWidth = 0;
  unsigned int tmpRound = 0;
  move_offset = 10;
  locPosition = (unsigned long)(AProgressBar->Position - AProgressBar->Min) * 100 / (AProgressBar->Max - AProgressBar->Min);
  if (AProgressBar->Visible == 1) {
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Background_Color, 0,0,0,0);
    TFT_Set_Pen(AProgressBar->Background_Color, AProgressBar->Pen_Width);
    TFT_Write_Text_Return_Pos(AProgressBar->Caption, AProgressBar->Left, AProgressBar->Top);
    if (caption_height >  AProgressBar->Height)
        caption_height =  AProgressBar->Height;
    if (caption_length >  AProgressBar->Width)
        caption_length =  AProgressBar->Width;
    if (AProgressBar->Position < (AProgressBar->Left + (AProgressBar->Width - caption_length) / 2)) {
    TFT_Rectangle(AProgressBar->Left + (AProgressBar->Width - caption_length) / 2,
                  AProgressBar->Top +(AProgressBar->Height - caption_height) / 2,
                  AProgressBar->Left + (AProgressBar->Width - caption_length) / 2 + caption_length,
                  AProgressBar->Top +(AProgressBar->Height - caption_height) / 2 + caption_height);
    }
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
    TFT_Set_Pen(AProgressBar->Pen_Color, AProgressBar->Pen_Width);
    if (AProgressBar->Rounded == 1) {
      if (locPosition > 0) {
         // Then position rect
         TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
         TFT_Rectangle(AProgressBar->Left + AProgressBar->Corner_Radius, AProgressBar->Top,  AProgressBar->Left + AProgressBar->Corner_Radius  + locPosition *  (AProgressBar->Width-AProgressBar->Corner_Radius*2) / 100 , AProgressBar->Top + AProgressBar->Height);
      }
    }
    else {
      if (locPosition > 0) {
         TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
         TFT_Rectangle(AProgressBar->Left, AProgressBar->Top,  AProgressBar->Left + (locPosition * AProgressBar->Width) / 100, AProgressBar->Top + AProgressBar->Height);
      }
    }
    // Draw not smoothed position rect
    if (AProgressBar->Smooth != 1) {
       if(AProgressBar->Rounded == 1) {
          tmpRound = AProgressBar->Corner_Radius;
          cnt = move_offset + tmpRound;
          tmpWidth = AProgressBar->Width - AProgressBar->Corner_Radius * 2;
       }
       else {
          tmpRound = 0;
          cnt = move_offset;
          tmpWidth = AProgressBar->Width;
       }
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Background_Color, 0,0,0,0);
        while (cnt-tmpRound < (locPosition * tmpWidth) / 100) {
              if  (cnt - tmpRound + 3 >=  tmpWidth * locPosition)
TFT_Rectangle(AProgressBar->Left + cnt, AProgressBar->Top, AProgressBar->Left + tmpRound  + tmpWidth * locPosition , AProgressBar->Top + AProgressBar->Height);
              else
          TFT_Rectangle(AProgressBar->Left + cnt, AProgressBar->Top,  AProgressBar->Left + cnt + 3, AProgressBar->Top + AProgressBar->Height);
          cnt = cnt + move_offset + 3;
        }
      }
    // Clear caption frist
    if (AProgressBar->Show_Position == 1){
       if (AProgressBar->Show_Percentage == 1) {
          WordToStr(locPosition , tmpStr);
          DeleteTrailingSpaces(tmpStr);
          strcpy(AProgressBar->Caption, tmpStr);
          strcat(AProgressBar->Caption, "\%");
    }
    else {
         WordToStr(AProgressBar->Position , tmpStr);
         DeleteTrailingSpaces(tmpStr);
         strcpy(AProgressBar->Caption, tmpStr);
    }
    TFT_Set_Font(AProgressBar->FontName, AProgressBar->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text_Return_Pos(AProgressBar->Caption, AProgressBar->Left, AProgressBar->Top);
    TFT_Write_Text(AProgressBar->Caption, AProgressBar->Left + (AProgressBar->Width - caption_length) / 2, AProgressBar->Top + (AProgressBar->Height - caption_height) / 2);
    }
 }
}

void DrawProgressBar(TProgressBar *AProgressBar) {
  unsigned long locPosition = 0;
  int move_offset = 0;
  unsigned cnt = 0;
  unsigned int tmpWidth = 0;
  unsigned int tmpRound = 0;
  unsigned int _tmpI, _tmpII;
  char tmpStr[8];
  move_offset = 10;
  _tmpI = AProgressBar->Position - AProgressBar->Min;
  _tmpII = AProgressBar->Max - AProgressBar->Min;
  locPosition = (unsigned long)_tmpI * 100 / _tmpII;
  if (AProgressBar->Visible == 1) {
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Background_Color, 0,0,0,0);
    TFT_Set_Pen(AProgressBar->Pen_Color, AProgressBar->Pen_Width);
    if (AProgressBar->Rounded == 1) {
      // First draw back rect
      TFT_Rectangle_Round_Edges(AProgressBar->Left, AProgressBar->Top,  AProgressBar->Left + AProgressBar->Width, AProgressBar->Top + AProgressBar->Height, AProgressBar->Corner_Radius);
      if (locPosition > 0) {
         // Then position rect
         TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
         TFT_Rectangle(AProgressBar->Left + AProgressBar->Corner_Radius, AProgressBar->Top,  AProgressBar->Left + AProgressBar->Corner_Radius   + locPosition *  (AProgressBar->Width-AProgressBar->Corner_Radius*2) / 100, AProgressBar->Top + AProgressBar->Height);
      }
    }
    else {
      // First draw back rect
      TFT_Rectangle(AProgressBar->Left, AProgressBar->Top, AProgressBar->Left + AProgressBar->Width, AProgressBar->Top + AProgressBar->Height);
      if (locPosition > 0) {
         // Then position rect
         TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
         TFT_Rectangle(AProgressBar->Left, AProgressBar->Top,  AProgressBar->Left + (locPosition * AProgressBar->Width) / 100, AProgressBar->Top + AProgressBar->Height);
      }
    }
    // Draw not smoothed position rect
    if (AProgressBar->Smooth != 1) {
       if(AProgressBar->Rounded == 1) {
          tmpRound = AProgressBar->Corner_Radius;
          cnt = move_offset + tmpRound;
          tmpWidth = AProgressBar->Width - AProgressBar->Corner_Radius * 2;
       }
       else {
          tmpRound = 0;
          cnt = move_offset;
          tmpWidth = AProgressBar->Width;
       }
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Background_Color, 0,0,0,0);
     while (cnt-tmpRound < (locPosition * tmpWidth) / 100) {
            if  (cnt - tmpRound + 3 >=  tmpWidth * locPosition)
              TFT_Rectangle(AProgressBar->Left + cnt, AProgressBar->Top, AProgressBar->Left + tmpRound  + tmpWidth * locPosition , AProgressBar->Top + AProgressBar->Height);
            else
              TFT_Rectangle(AProgressBar->Left + cnt, AProgressBar->Top,  AProgressBar->Left + cnt + 3, AProgressBar->Top + AProgressBar->Height);
        cnt = cnt + move_offset + 3;
     }
    }
    // Clear caption first
    if (AProgressBar->Show_Position == 1){
       if (AProgressBar->Show_Percentage == 1) {
          WordToStr(locPosition , tmpStr);
          DeleteTrailingSpaces(tmpStr);
          strcpy(AProgressBar->Caption, tmpStr);
          strcat(AProgressBar->Caption, "\%");
       }
       else {
         WordToStr(AProgressBar->Position , tmpStr);
         DeleteTrailingSpaces(tmpStr);
         strcpy(AProgressBar->Caption, tmpStr);
       }
       TFT_Set_Font(AProgressBar->FontName, AProgressBar->Font_Color, FO_HORIZONTAL);
       TFT_Write_Text_Return_Pos(AProgressBar->Caption, AProgressBar->Left, AProgressBar->Top);
       TFT_Write_Text(AProgressBar->Caption, AProgressBar->Left + (AProgressBar->Width - caption_length) / 2, AProgressBar->Top + (AProgressBar->Height - caption_height) / 2);
     }
 }
}

void DrawScreen(TScreen *aScreen) {
 unsigned int order;
  unsigned short button_idx;
  TButton *local_button;
  unsigned short label_idx;
  TLabel *local_label;
  unsigned short progress_bar_idx;
  TProgressBar *local_progress_bar;
  char save_bled, save_bled_direction;

  object_pressed = 0;
  order = 0;
  button_idx = 0;
  label_idx = 0;
  progress_bar_idx = 0;
  CurrentScreen = aScreen;

  if ((display_width != CurrentScreen->Width) || (display_height != CurrentScreen->Height)) {
    save_bled = TFT_BLED;
    TFT_BLED           = 0;
    TFT_Init_ILI9341_8bit(CurrentScreen->Width, CurrentScreen->Height);
    TP_TFT_Init(CurrentScreen->Width, CurrentScreen->Height, 8, 9);                                  // Initialize touch panel
    TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold
    TFT_Fill_Screen(CurrentScreen->Color);
    display_width = CurrentScreen->Width;
    display_height = CurrentScreen->Height;
    TFT_BLED           = save_bled;
  }
  else
    TFT_Fill_Screen(CurrentScreen->Color);


  while (order < CurrentScreen->ObjectsCount) {
    if (button_idx < CurrentScreen->ButtonsCount) {
      local_button = GetButton(button_idx);
      if (order == local_button->Order) {
        button_idx++;
        order++;
        DrawButton(local_button);
      }
    }

    if (label_idx < CurrentScreen->LabelsCount) {
      local_label = GetLabel(label_idx);
      if (order == local_label->Order) {
        label_idx++;
        order++;
        DrawLabel(local_label);
      }
    }

    if (progress_bar_idx < CurrentScreen->ProgressBarsCount) {
      local_progress_bar = GetProgressBar(progress_bar_idx);
      if (order == local_progress_bar->Order) {
        progress_bar_idx++;
        order++;
        DrawProgressBar(local_progress_bar);
      }
    }

  }
}

void Get_Object(unsigned int X, unsigned int Y) {
  button_order        = -1;
  label_order         = -1;
  //  Buttons
  for ( _object_count = 0 ; _object_count < CurrentScreen->ButtonsCount ; _object_count++ ) {
    local_button = GetButton(_object_count);
    if (local_button->Active == 1) {
      if (IsInsideObject(X, Y, local_button->Left, local_button->Top,
                         local_button->Width, local_button->Height) == 1) {
        button_order = local_button->Order;
        exec_button = local_button;
      }
    }
  }

  //  Labels
  for ( _object_count = 0 ; _object_count < CurrentScreen->LabelsCount ; _object_count++ ) {
    local_label = GetLabel(_object_count);
    if (local_label->Active == 1) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  _object_count = -1;
  if (button_order >  _object_count )
    _object_count = button_order;
  if (label_order >  _object_count )
    _object_count = label_order;
}


void Process_TP_Press(unsigned int X, unsigned int Y) {
  exec_button         = 0;
  exec_label          = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == button_order) {
      if (exec_button->Active == 1) {
        if (exec_button->OnPressPtr != 0) {
          exec_button->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnPressPtr != 0) {
          exec_label->OnPressPtr();
          return;
        }
      }
    }

  }
}

void Process_TP_Up(unsigned int X, unsigned int Y) {

  switch (PressedObjectType) {
    // Button
    case 0: {
      if (PressedObject != 0) {
        exec_button = (TButton*)PressedObject;
        if ((exec_button->PressColEnabled == 1) && (exec_button->OwnerScreen == CurrentScreen)) {
          DrawButton(exec_button);
        }
        break;
      }
      break;
    }
  }

  exec_label          = 0;

  Get_Object(X, Y);


  if (_object_count != -1) {
  // Buttons
    if (_object_count == button_order) {
      if (exec_button->Active == 1) {
        if (exec_button->OnUpPtr != 0)
          exec_button->OnUpPtr();
        if (PressedObject == (void *)exec_button)
          if (exec_button->OnClickPtr != 0)
            exec_button->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Labels
    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnUpPtr != 0)
          exec_label->OnUpPtr();
        if (PressedObject == (void *)exec_label)
          if (exec_label->OnClickPtr != 0)
            exec_label->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  }
  PressedObject = 0;
  PressedObjectType = -1;
}

void Process_TP_Down(unsigned int X, unsigned int Y) {

  object_pressed      = 0;
  exec_button         = 0;
  exec_label          = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == button_order) {
      if (exec_button->Active == 1) {
        if (exec_button->PressColEnabled == 1) {
          object_pressed = 1;
          DrawButton(exec_button);
        }
        PressedObject = (void *)exec_button;
        PressedObjectType = 0;
        if (exec_button->OnDownPtr != 0) {
          exec_button->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        PressedObject = (void *)exec_label;
        PressedObjectType = 2;
        if (exec_label->OnDownPtr != 0) {
          exec_label->OnDownPtr();
          return;
        }
      }
    }

  }
}

void Check_TP() {
  if (TP_TFT_Press_Detect()) {
    if (TP_TFT_Get_Coordinates(&Xcoord, &Ycoord) == 0) {
    // After a PRESS is detected read X-Y and convert it to Display dimensions space
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
}

void Init_MCU() {
  GPIO_Digital_Output(&GPIOE_BASE, _GPIO_PINMASK_9);
  TFT_BLED = 1;
  TFT_Set_Default_Mode();
  TP_TFT_Set_Default_Mode();
}

void Start_TP() {
  Init_MCU();

  InitializeTouchPanel();

  // You can get calibration constants using touch panel calibration example
  TP_TFT_Set_Calibration_Consts(239, 3700, 224, 3679);    // Set calibration constants

  InitializeObjects();
  display_width = Screen1.Width;
  display_height = Screen1.Height;
  DrawScreen(&Screen1);
}
